@@ -122,7 +122,15 @@ class DataSourceHal(DataSource):
 
         ret = {}
         for k in changedAndAvailable:
-            ret[k] = available[k]
+            if self.lastValues[k][0] != available[k]:
+                # We still only want to record the value if it has really
+                # changed since our last observation. Because of our
+                # 'greedy' data collection method, we sometimes have already
+                # recorded the changed value before it is announced by a
+                # DBUS signal.
+                # Also note that most of the rest of this method handles
+                # exceptions to the rule stated above.
+                ret[k] = available[k]
 
         now = int(time.time())
 
@@ -130,7 +138,9 @@ class DataSourceHal(DataSource):
         for k in available.keys():
             if k in ('hal.battery.charge_level.percentage',
                      'hal.battery.reporting.current',
-                     'hal.battery.voltage.current'):
+                     'hal.battery.voltage.current',
+                     'hal.battery.reporting.design',
+                     'hal.battery.voltage.design'):
                 if self.lastValues[k][1] < now - 900 \
 			and self.lastValues[k][0] != available[k]:
                     additional.append(k)
@@ -147,20 +157,26 @@ class DataSourceHal(DataSource):
         if available.has_key(chrg) and available.has_key(dchrg) \
                 and available[chrg] and not available[dchrg]:
             # Don't return percentage and energy, they're not being
-            # updated during charge
+            # updated during charge...
             if (self.lastValues[dchrg][0] == True):
-                # Except, if we just started charging. Then we want to record
+                # ..except if we just started charging. Then we want to record
                 # the last known values.
-                # (This should fix charge time start inaccuracy)
+                # (This should fix the charge that appears to begin too early)
                 for key in ('hal.battery.charge_level.percentage',
                             'hal.battery.reporting.current'):
                     if not ret.has_key(key) and available.has_key(key):
                         ret[key] = available[key]
             else:
                 # Don't record
-                ret.pop('hal.battery.charge_level.percentage', None)
-                ret.pop('hal.battery.reporting.current', None)
-
+                for key in ('hal.battery.charge_level.percentage',
+                            'hal.battery.reporting.current'):
+                    # Remove from returned dict
+                    ret.pop(key, None)
+                    # Also mark it as updated with it's latest value, so we
+                    # don't record it until it's properly recalculated at
+                    # end of charge [bug 5235]
+                    if available.has_key(key):
+                        self.markUpdated(key, available[key], now)
         return ret 
 
     def dictionarize(self, lines):
