@@ -18,6 +18,9 @@
 
 import sqlite3
 
+# profiling
+import time
+
 class DataStorage(object):
     def __init__(self, path, dataSources):
         self.path = path
@@ -48,6 +51,7 @@ class DataStorage(object):
         self.con.commit()
 
     def __getObservations(self, cursor, type, start, end):
+        #t1 = time.time()
         if isinstance(type, (tuple, list)):
             cursor.execute('''select timestamp, value, observation_type.name
                               from observation
@@ -66,39 +70,29 @@ class DataStorage(object):
                               and timestamp between ? and ?
                               order by timestamp asc''',
                               (type, start, end))
+        #print "Query: %.3fs" % (time.time() - t1)
 
     def getObservations(self, type, start, end):
         cursor = self.con.cursor()
         self.__getObservations(cursor, type, start, end)
-        ret = []
-        for row in cursor.fetchall():
+        def intOrNone(input):
             try:
-                ret.append((int(row[0]), int(row[1]), row[2]))
+                return int(input)
             except TypeError:
-                if row[1] == None:
-                    ret.append((int(row[0]), None, row[2]))
-                else:
-                    raise TypeError("Query returned unknown data")
-        return ret
+                return None
+        return map(lambda row: (row[0], intOrNone(row[1]), row[2]), cursor.fetchall())
 
     def getObservationsWithBreaks(self, type, start, end):
         cursor = self.con.cursor()
-
-        if isinstance(type, (tuple, list)):
-            type = tuple(type) + ('internal.graph.break')
-        else:
-            type = (type, 'internal.graph.break')
-
-        self.__getObservations(cursor, type, start, end)
         ret = []
-        segment = []
-        ret.append(segment)
-        for row in cursor.fetchall():
-            if row[2] == 'internal.graph.break':
-                segment = []
-                ret.append(segment)
-            else:
-                segment.append((int(row[0]), int(row[1]), row[2]))
+        self.__getObservations(cursor, 'internal.graph.break', start, end)
+        breaks = cursor.fetchall()
+        breaks.append((end+1, None, 'internal.graph.break'))
+        intervalStart = start
+        for b in breaks:
+            self.__getObservations(cursor, type, intervalStart+1, b[0]-1)
+            ret.append(map(lambda row: (row[0], int(row[1]), row[2]), cursor.fetchall()))
+            intervalStart = b[0]
         return ret
 
     def getPreviousObservationTime(self, timestamp):
@@ -141,8 +135,10 @@ class DataStorage(object):
 
                 unique (name)
             )''']
+        indexes = ['timestamp_index on observation (timestamp)']
         cursor = self.con.cursor()
         for table in tables:
             cursor.execute('create table if not exists %s' % table)
-
+        for index in indexes:
+            cursor.execute('create index if not exists %s' % index)
 
